/*
 * PPM Image Library
 * by Joel R. Voss
 * 09/09/2004
 */

#include "image.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

// Constructor for a color.
struct Color setColor(BYTE init_r, BYTE init_g, BYTE init_b)
{
	struct Color c;
	c.r = init_r;
	c.g = init_g;
	c.b = init_b;
	return c;
}

struct Image createImage(int init_width, int init_height, int init_bpp)
{
	struct Image image;
	image.height	= init_height;
	image.width		= init_width;
	image.colors	= 255; // No reason to allow them to set this.
	// Colors is needed for PNM file.
	image.bpp		= init_bpp; // Bits per pixel = 24 or 8.
	
	// Allocate the data for the image.
	image.data		= (BYTE *)malloc(init_width*init_height*init_bpp/8);
	return image;
}

// This is a simple way to grab height and width from a PNM file while ensuring 
// comments do not get in the way.
// Specifically, The GIMP creates PPM files that require this type of function.
int getInt(FILE *inFile)
{
	// Line size is limited to 1024. Hopefully no one tries to fuxx0r with that.
	// FIXME: This is vulnerable to a buffer overflow.
	char sTemp[1024];
	
	// Get a word.
	fscanf(inFile, "%s", sTemp);
	// If it's a comment, ignore.
	while(sTemp[0] == '#' && !feof(inFile))
	{
		// Next line.
		fgets(sTemp, 1024, inFile);
		// Get another word.
		fscanf(inFile, "%s", sTemp);
	}
	// If it's not a comment, it must be an int.
	return atoi(sTemp);
}

// The image loader. 
// Accepts PNM (PPM and PGM, P5 and P6, RAW, not ASCII)
struct Image loadImage(const char *sFile)
{
	struct Image image;
	
	FILE *inFile = fopen(sFile, "r");
	if(inFile == 0)
	{
		image.width  = 0;
		image.height = 0;
		image.colors = 0;
		image.bpp    = 0;
		image.data   = 0;
		printf("Image File Error.\n");
		return image;
	}
	
	char sHeader[10];
	// Header = P5 or P6 depending on bits.
	fgets(sHeader, 10, inFile);
	// Width is an integer.
	image.width  = getInt(inFile);
	// Height is an integer.
	image.height = getInt(inFile);
	// Colors is an integer, usually 255.
	image.colors = getInt(inFile);
	if(0 == strncmp(sHeader, "P6", 2))
	{
		image.bpp = 24;
	}
	else if(0 == strncmp(sHeader, "P5", 2))
	{
		image.bpp = 8;
	}
	else
	{
		image.width  = 0;
		image.height = 0;
		image.colors = 0;
		image.bpp    = 0;
		image.data   = 0;
		printf("Image Header Error.\n");
		return image;
	}
	// Data is just an array.
	image.data = (BYTE *)malloc(image.height*image.width*image.bpp/8);
	
	// delete the fricking carraige return.
	char cr;
	fread(&cr, 1, 1, inFile);
	
	// Get the data
	fread(image.data, image.height*image.width*image.bpp/8, 1, inFile);
	// Close the input file.
	fclose(inFile);
	// Return the image.
	return image;
}

// This is an unused function for drawing pixels one at a time.
void drawImage(struct Image *image, int x, int y, struct Color c)
{
	if(x < image->width && y < image->height && x >=0 && y >=0)
	{
		int stride = image->width*image->bpp/8;
		image->data[x + y*stride]     = c.r;
		if(image->bpp == 24)
		{
			image->data[x + y*stride + 1] = c.g;
			image->data[x + y*stride + 2] = c.b;
		}
	}
	else
	{
		printf("draw() error: %i, %i;\n", x, y);
	}
}

// This saves the image in PNM format.
void saveImage(struct Image *image, const char *sFile)
{
	FILE *outFile = fopen(sFile, "w");
	fprintf(outFile, "P%i\n%i %i %i\n", ((image->bpp == 24) ? 6 : 5), image->width, image->height, image->colors);
	fwrite((char *)image->data, image->width * image->height*image->bpp/8, 1, outFile);
	fclose(outFile);
}

#ifdef TEST

#include <stdio.h>

// Test image load library
int main(int argc, char **argv)
{
	struct Image iTest;
	iTest = loadImage("test1.pnm");
	saveImage(&iTest, "test1a.pnm");
	
	free(iTest.data);
	
	//void drawImage(struct Image *image, int x, int y, struct Color c);
	//struct Image createImage(int init_height, int init_width, int init_bpp);
	
	return 0;
}
#endif // TEST
