/*
 * Rubber Spoon libSteg 0.4
 * by Joel R. Voss
 * August 16, 2004 10:55
 * SEE rs04.txt for more details.
 */

#ifndef RS043_H_

#define RS043_H_

#include "types.h"

// 1a) data bits -> data byte
// convertBitToByte converts one bit to a byte with the bit in the lowest 
// position. if bit is true, 1, else 0.
BYTE convertBitToByte(BYTE input, BYTE bit);

// 1b) data bits -> data byte
// convertHexToBinary converts a byte to 8 bytes with each bit in the lowest 
// position of the byte it created.
void convertHexToBinary(BYTE data, BYTE *dest);

// 1c) data bits -> data byte
// convertDataToBinary converts a N bytes to 8 * N bytes with each bit in the 
// lowest position of the byte it created.
void convertDataToBinary(BYTE *data, DWORD size, BYTE *dest);

// Key Binary Is the data stored into the keymask.
// When steg() calls orData(data, keyBinary), it keys it.
void convertDataToKeyBinary(BYTE *data, DWORD size, int *key, DWORD key_size, BYTE *dest);

// Stegkey turns the password into the keyhash using SHA1.
// Output is in the format: 1,7,12,16,22,24,29,33,34,...
void StegKey(char *SecretCode, int *KeyHash);

// StegNoKey outputs 123456...50.
// Useful for most efficient keyless system.
void StegNoKey(int *KeyHash);

// 2) Mask
// createMask will become an ultimate mask generator system.
// for now, it simply writes size bytes of 0x1 = binary 00000001.
void createMask(BYTE *dest, BYTE data, DWORD size);

// The KeyMask puts ff where there is no key, fe where there is a key.
// data AND fe deletes the least significant bit of data.
void createKeyMask(int *key, DWORD key_size, BYTE *dest);

// 3a) dest1 = pixels and inverse mask
// andData ands all the data in a and b.
//void andData(BYTE *a, BYTE *b, DWORD size, BYTE *dest);
void andData(BYTE *a, DWORD asize, BYTE *b, DWORD bsize, BYTE *dest, DWORD size);

// 3b) dest2 = dest1 or data bytes
// orData ors all the data in a and b.
//void orData(BYTE *a, BYTE *b, DWORD size, BYTE *dest);
void orData(BYTE *a, DWORD asize, BYTE *b, DWORD bsize, BYTE *dest, DWORD size);

// steg: A simple function to take data and interlace it with pixels of an 
// image such that an onlooker will not see the data. <1>
//void steg(BYTE *data, DWORD size, BYTE *pixels, BYTE *output);
void steg(BYTE *data, DWORD size, int *key, DWORD key_size, BYTE *pixels, BYTE *output);


/* <2> For example, if I had a stegged picture with a good key, a person could 
 * brute force the key and get several messages: one might say "meet me at 6" 
 * another might say, "terror at 7 pm", while another might say "i R 1337". 
 * Which is the most likely? Probably the third.
 */

#endif // RS043_H_
