/*
 * Rubber Spoon: LibSteg 0.4.3
 * by Joel R. Voss
 *
 */

// libSteg 0.4.3 steg
#include "rs043.h"

// readFile function is useful..
#include "readfile.h"

// Image load, edit, and save
// Only accepts ppm and pgm (24-bit and 8-bit raw pnm files)
#include "image.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void usage()
{
	printf("steg04 file.pnm data.txt stegout.pnm\n");
}


int main(int argc, char **argv)
{
	printf("Rubber Spoon: LibSteg 0.4.2 Test 2\n");
	
	if(argc < 4)
	{
		usage();
		return 1;
	}
	BYTE *data = 0;

	// See image.h, image.c
	struct Image image, output;
	
	// The secret data.
	int dataSize  = readFile(argv[2], &data);
	
	// SHA Hash Key
	int key[50];
	
	// Secret Key
	// TODO: user input
	StegKey("javantea", key);
	
	// The Portable aNyMap
	image = loadImage(argv[1]);
	int pixelSize = image.height * image.width * image.bpp / 8;
	
	// bytes = size * 8 bits * key[key_size - 1] per 50 bits
	int realSize = dataSize * 8 * (key[49]+1) / 50;
	
	printf("sizes: %i, %i, %i\n", dataSize, realSize, pixelSize);
	
	// Do not allow any missing data or memory leak.
	if(data == 0 || dataSize <= 0 || pixelSize <= 0 || pixelSize < realSize)
	{
		printf("Error: Data or Portable aNyMap is null.\n");
		free(image.data);
		free(data);
		return 1;
	}
	
	// The output image is a blank sheet the same size as the input image.
	output = createImage(image.width, image.height, image.bpp);
	
	// Ensure that the data is allocated.
	if(output.data == 0)
	{
		printf("Error: Memory allocation failed.\n");
		free(image.data);
		free(data);
		return 1;
	}
	
	// The actual steganography.
	// steg(secret, secret_size, key, key_size, image pixels, output pixels);
	steg(data, dataSize, key, 50, image.data, output.data);

	// Steg does not fill in the bytes past the stegged part, 
	// so do that manually.
	int offset = realSize;
	int fill = pixelSize - offset;
	memcpy(&(output.data[offset]), &(image.data[offset]), fill);

	// Save the resulting output to the file specified.
	saveImage(&output, argv[3]);

	// Must deallocate the memory for all the arrays.
	free(output.data);
	free(image.data);
	free(data);
	return 0;
}
